Translation.addTranslation("Mineral Digging: Active", {zh: "连锁挖矿：开启"});
Translation.addTranslation("Mineral Digging: Disabled", {zh: "连锁挖矿：关闭"});

IMPORT("ToolLib");

var LocalConfig = {
    enabled: false,
    radius: Number(__config__.getNumber("radius"))
}, ConfigList = {},
Max_radius = Number(__config__.getNumber("server.max_radius")),
DropToPlayer = Boolean(__config__.getBool("server.drop2player"));

//run on server
Network.addServerPacket("MineralDigging_Server_setConfig", function(client, T){
	if(T.radius > Max_radius) T.radius = Max_radius;
	ConfigList[client.getPlayerUid()] = T;
});
//run on client
Network.addClientPacket("MineralDigging_Client_int", function(T){
	Network.sendToServer("MineralDigging_Server_setConfig", LocalConfig);
});


Callback.addCallback("ServerPlayerLoaded", function(player){
	Network.getClientForPlayer(player).send("MineralDigging_Client_int", {});
});
Callback.addCallback("LevelLeft", function(){
	ConfigList = {};
});


var settingUI = {
    window: new UI.Window({
        location: {x: 600, y: 0, width: 50, height: 40},
        drawing: [{type: "background", color: android.graphics.Color.TRANSPARENT}],
        elements: {
            "close": {type: "button", x: 0, y: 400/3, scale: 1000/30, bitmap: "default_switch_off", clicker: {
                onClick: function(){
                		try{
                			LocalConfig.enabled = !LocalConfig.enabled;
                			settingUI.window.getContent().elements.close.bitmap = LocalConfig.enabled ? "default_switch_on" : "default_switch_off";
                			alert(Translation.translate("Mineral Digging: "+(LocalConfig.enabled ? "Active" : "Disabled")));
                			Network.sendToServer("MineralDigging_Server_setConfig", LocalConfig);
                		}catch(e){};
                	}
                }
            }
        }
    }),
    open: function(){
    	this.window.setAsGameOverlay(true);
    	this.window.open();
    },
    close: function(){this.window.close()},
};

Callback.addCallback("NativeGuiChanged", function(screenName){
	if(screenName=="in_game_play_screen" || screenName=="hud_screen"){
		settingUI.open();
	}else{
		settingUI.close();
	}
});



var SpecialDrop = {};
var DIRS=[];
for(var x = -1; x <= 1; x++){
	for(var y = -1; y <= 1; y++){
		for(var z = -1; z <= 1; z++){
			if(!(x==0 && y==0 && z==0)){
				DIRS.push({x: x, y: y, z: z});
			}
		}
	}
};

function addSpecialDrop(id, func){
	if(!id || !func) return;
	SpecialDrop[id] = func;
}

function Distance(coords, center){
	return {
	    x: Math.abs(coords.x-center.x),
	    y: Math.abs(coords.y-center.y),
	    z: Math.abs(coords.z-center.z)
	};
}

function DropItem(coords, item, player, blockSource){
	if(DropToPlayer && player){
		var p = Entity.getPosition(player);
		blockSource.spawnDroppedItem(p.x, p.y, p.z,
		    item.id, item.count, item.data, item.extra || null);
	} else {
		blockSource.spawnDroppedItem(coords.x+0.5, coords.y+0.5, coords.z+0.5,
		    item.id, item.count, item.data, item.extra || null);
	}
}

function Destroy(coords, block, player, item){
	if(Entity.getCarriedItem(player).id != item.id) return;
	var blockSource = BlockSource.getDefaultForActor(player);
	if(SpecialDrop[block.id]){
		SpecialDrop[block.id](coords, block, player, blockSource);
	} else {
		var block_drop = ToolLib.getBlockDrop(coords, block.id, block.data,
		    ToolAPI.getToolLevelViaBlock(item.id, block.id), ToolAPI.getEnchantExtraData(item.extra), item, blockSource) || [];
		block_drop.map(function(v, k){
			if(v[0]) DropItem(coords, {id: v[0], count: v[1], data: v[2], extra: v[3]}, player, blockSource);
		});
	}
	Callback.invokeCallback("DestroyBlock", coords, block, player, true);
	blockSource.setBlock(coords.x, coords.y, coords.z, 0, 0);
}

function DestroyBlock(origin, center, coords, player, item, object){
	if(!ConfigList[player]) return;
	var blockSource = BlockSource.getDefaultForActor(player);
	var block = blockSource.getBlock(coords.x, coords.y, coords.z);
	var coords_dis = Distance(center, coords);
	if(block.id == origin.id && block.data == origin.data && coords.y >= 0 && coords.y <= 255){
		Destroy(coords, block, player, item);
		object[coords.x+":"+coords.y+":"+coords.z] = true;
		DIRS.map(function(v, k){
			var next = {x: coords.x-(0-v.x), y: coords.y-(0-v.y), z: coords.z-(0-v.z)};
			var next_dis = Distance(center, next);
			if(!object[next.x+":"+next.y+":"+next.z]
			    && next_dis.x <= ConfigList[player].radius
			    && next_dis.y <= ConfigList[player].radius
			    && next_dis.z <= ConfigList[player].radius){
				DestroyBlock(origin, center, next, player, item, object);
			}
		})
	}
}

Callback.addCallback("DestroyBlock", function (coords, block, player, auto){
	if(auto) return;
	if(block.id && ConfigList[player] && ConfigList[player].enabled && ConfigList[player].radius > 0){
		var object = {};
		object[coords.x+":"+coords.y+":"+coords.z] = true;
		DIRS.map(function(v, k){
			DestroyBlock(block, coords, {
			    x: coords.x-(0-v.x),
			    y: coords.y-(0-v.y),
			    z: coords.z-(0-v.z),
			}, player, Entity.getCarriedItem(player), object);
		});
	}
});



addSpecialDrop(17, function(coords, block, player, blockSource){
	DropItem(coords, {id: 17, count: 1, data: block.data%4}, player, blockSource);
});
addSpecialDrop(162, function(coords, block, player, blockSource){
	DropItem(coords, {id: 162, count: 1, data: block.data%4}, player, blockSource);
});


ModAPI.registerAPI("MineralDiggingAPI", {
    version: [1, 2],
    dropItem: DropItem,
    addSpecialDrop: addSpecialDrop
});
Logger.Log("The API of Mineral Digging is named MineralDiggingAPI.", "API");

